/**
 * Fichero : tweets.c      Fecha:23/12/2016
 * Autores:                                   
 *                                            
 *  fichero con la implementacion del array de tweets
 *                                            
 */

#include "tweets.h"
#include "const.h"

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

#define TAM_BLOQUE 10

/*********************************************** Estructura del TAD tweet ***/
struct _tweets {
	tweet **array_tweets;
	int n_tweets;
	int n_reserved;
	int last_id;
};

/****** Funciones internas *****/
BOOL has_id(tweets * tts, long long int id);
BOOL validate_capacity(tweets * tts);

/**
 * Funcion para crear una nueva estructura tweet 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura tweets inicializado todo a vacio o NULL en caso de error
 */
tweets * new_tweets()
{
	tweets *tts;

	tts = (tweets *) malloc(sizeof(tweets));

	if (tts == NULL) {
		return NULL;
	}

	tts-> array_tweets = (tweet **) malloc(TAM_BLOQUE * sizeof(tweet *));
	if (tts-> array_tweets == NULL) {
		free(tts);
		return NULL;
	}


	tts->n_reserved = TAM_BLOQUE;
	tts->last_id = 0;
	tts->n_tweets = 0;

	return tts;
}

/**
 * Funcion para liberar la memoria reservada de la estructura tweets y todos los
 *   tweets que estan en el array.
 *
 * Entrada:
 *    Estructura tweets a liberar
 */
void destroy_tweets(tweets *tts)
{

	assert(tts);

	if (tts->array_tweets != NULL) {
		free(tts->array_tweets);
	}

	free(tts);
}

/**
 * Funcion para aniadir un tweet ya creado al array 
 *
 * Entrada:
 *    Estructura del TAD tweets sobre el que se quiere aniadir un tweet
 *    Puntero al tweet a aniadir
 *
 * Salida:
 *    El numero de tweets del array o -1 en caso de error
 */
int add_tweet(tweets * tts, tweet * t)
{
	/*realloc*/
	assert(tts != NULL);

	if (t == NULL) {
		return NULL_ID;
	}

	if (!validate_capacity(tts)) {
		return NULL_ID;
	}

	if (get_tweet_id(t) == NULL_ID) {
		set_tweet_id(t, tts->last_id + 1);
		tts->last_id++;
	}

	if (has_id(tts, get_tweet_id(t))) {
		/*No permitimos aniadir tweets con la misma ID*/
		return NULL_ID;
	}

	tts->array_tweets[tts->n_tweets] = t;
	tts->n_tweets++;

	return get_tweet_id(t);
}

/**
 * Funcion para recuperar el numero de tweets del array
 *
 * Entrada:
 *    Estructura del TAD tweets sobre el que se quiere recuperar su tamanio
 *
 * Salida:
 *    El numero de tweets
 */
int get_n_tweets(tweets * tts)
{
	if (tts == NULL) {
		return -1;
	}

	return tts->n_tweets;
}

/**
 * Funcion para recuperar el tweet n-esimo de la lista
 *
 * Entrada:
 *    Estructura tweets e indice del tweet a recuperar (indexado sobre 0)
 *
 * Salida:
 *    Un puntero al tweet o NULL si hay un error
 */
tweet * get_tweet_i(tweets * tts, int index)
{
	if (tts == NULL || index >= tts->n_tweets || index < 0) {
		return NULL;
	}

	return tts->array_tweets[index ];

	/*Indexando sobre 1:*/
	/*
	if(tts == NULL || index > tts->n_tweets || index <= 0){
	    return NULL;
	}
    
	return tts->array_tweets[index - 1];*/
}

/**
 * Funcion para recuperar un tweet dado su código.
 *
 * Entrada:
 *    Colección de tweets y el código del tweet a recuperar.
 *
 * Salida:
 *    Un puntero al tweet o NULL si no se encuentra.
 */
tweet * get_tweet_by_id(tweets *tts, long long int id)
{
	int i;
	tweet *t;

	if (tts == NULL || id == NULL_ID) {
		return NULL;
	}

	t = NULL;
	for (i = 0; i < tts->n_tweets && t == NULL; i++) {
		if (get_tweet_id(tts->array_tweets[i]) == id) {
			t = tts->array_tweets[i];
		}
	}

	return t;
}

/**
 * Funcion para imprimir la lista de tweets por pantalla
 *
 * Entrada:
 *    Estructura TAD tweet a imprimir
 */
void print_tweets(tweets *tts)
{
	int i;

	if (tts == NULL) {
		printf("NULL\n");
		return;
	}

	printf("-----Coleccion de tweets-----\n");
	printf("Numero de tweets: %d\n", tts->n_tweets);
	printf("Espacio reservado para: %d tweets\n", tts->n_reserved);
	printf("Ultima ID: %d\n", tts->last_id);
	printf("-----------------------------\n\n");


	for (i = 0; i < tts->n_tweets; i++) {

		print_tweet(tts->array_tweets[i]);

	}

	printf("-----------------------------\n\n");
}

/**
 * Funcion para comprobar si la id está en la colección de tweets.
 *
 * Entrada:
 *    Colección de tweets y el código del tweet a comprobar
 *
 * Salida:
 *    TRUE si se encuentra la id. FALSE en caso contrario.
 */

BOOL has_id(tweets * tts, long long int id)
{
	int i;

	assert(tts != NULL && id != NULL_ID);

	for (i = 0; i < tts->n_tweets; i++) {
		if (get_tweet_id(tts->array_tweets[i]) == id) {
			return TRUE;
		}
	}

	return FALSE;
}


/**
 * Funcion para proporcionar una capacidad correcta al array antes de insertar
 * Comprueba si hay hueco para un tweet más. Si no lo hay, redimensiona el array
 *
 * Entrada:
 *    Colección de tweets a validar
 *
 * Salida:
 *    TRUE si la capacidad es adecuada tras su ejecución. FALSE en caso contrario
 * */

BOOL validate_capacity(tweets * tts)
{

	tweet ** nuevo_array;

	if (tts == NULL) {
		return FALSE;
	}

	if (tts->n_reserved <= tts->n_tweets) {
		nuevo_array = (tweet **) realloc(tts->array_tweets, (tts->n_reserved + TAM_BLOQUE) * sizeof(tweet *));

		if (nuevo_array == NULL) {
			return FALSE;
		}

		tts->array_tweets = nuevo_array;
		tts->n_reserved += TAM_BLOQUE;
	}

	return TRUE;
}

