/**
 * Fichero : twittero.c      Fecha:23/12/2016
 * Autores:                                   
 *                                            
 *  fichero con la implementacion del TAD twittero
 *                                            
 */

#include "twittero.h"
#include "tweets.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>

/*********************************************** Estructura del TAD twittero ***/
struct _twittero {
	long long int id;
	char * name;
	char * alias;
	tweets * mis_tweets; /* Tweets */
};

/**
 * Funcion para crear una nueva estructura twittero 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura twittero inicializado todo a vacio o NULL en caso de error
 */
twittero * new_twittero()
{
	twittero *u;

	u = (twittero *) malloc(sizeof(twittero));
	if (u == NULL) {
		printf("Error de reserva de memoria\n");
		return NULL;
	}

	u->mis_tweets = new_tweets();

	if (u->mis_tweets == NULL) {
		free(u);
		return NULL;
	}

	u->id = NULL_ID;
	u->name = NULL;
	u->alias = NULL;

	return u;
}

/**
 * Funcion para liberar la memoria reservada de la estructura twittero
 *
 * Entrada:
 *    Estructura twittero a liberar
 */
void destroy_twittero(twittero *u)
{
	assert(u);

	destroy_tweets(u->mis_tweets);

	if (u->name != NULL) {
		free(u->name);
	}

	if (u->alias != NULL) {
		free(u->alias);
	}

	free(u);
}

/**
 * Funcion para asignar el id a un twittero.
 *
 * Entrada:
 *    Estructura twittero e id del mismo
 *
 */
void set_twittero_id(twittero * u, long long int id)
{
	assert(u);

	u->id = id;

}

/**
 * Funcion para asignar el nombre a un twittero
 *
 * Entrada:
 *    Estructura twittero y el nombre
 *
 */
void set_twittero_name(twittero * u, const char *name)
{
	char * nuevo_name;
	assert(u != NULL && name != NULL);

	/* Reservamos la longitud de la cadena mas 1 para el terminador nulo */
	nuevo_name = (char *) realloc(u->name, (strlen(name) + 1) * sizeof(char));

	if (nuevo_name == NULL) {
		return;
	}

	u->name = nuevo_name;

	strcpy(u->name, name);
}

/**
 * Funcion para asignar el alias a un twittero
 *
 * Entrada:
 *    Estructura twittero y el alias
 *
 */
void set_twittero_alias(twittero * u, const char *alias)
{
	char * nuevo_alias;
	assert(u != NULL && alias != NULL);

	/* Reservamos la longitud de la cadena mas 1 para el terminador nulo */
	nuevo_alias = (char *) realloc(u->alias, (strlen(alias) + 1) * sizeof(char));

	if (nuevo_alias == NULL) {
		return;
	}

	u->alias = nuevo_alias;

	strcpy(u->alias, alias);
}

/**
 * Funcion para recuperar el id de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre la que se quiere recuperar el id
 *
 * Salida:
 *    El id
 */
long long int get_twittero_id(twittero * u)
{
	assert(u);

	return u->id;
}

/**
 * Funcion para recuperar el nombre de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre el que se quiere recuperar el nombre
 *
 * Salida:
 *    El nombre del twittero  (es constante para que no se pueda modificar
 *        desde fuera)
 */
const char * get_twittero_name(twittero * u)
{
	assert(u);

	return u->name;

}

/**
 * Funcion para recuperar el alias de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre el que se quiere recuperar el alias
 *
 * Salida:
 *    El alias del twittero  (es constante para que no se pueda modificar
 *        desde fuera)
 */
const char * get_twittero_alias(twittero * u)
{
	assert(u);

	return u->alias;
}

/**
 * Funcion para añadir un tweet a un twittero 
 * No se comprueban duplicados (i.e., que el mismo tweet sea aniadido dos veces)
 *
 * Entrada:
 *    El twittero y el tweet que se quiere aniadir.
 *
 * Salida:
 *    
 */
void add_twittero_tweet(twittero *user, tweet *t)
{
	assert(user != NULL && t != NULL);

	set_tweet_author(t, user);

	add_tweet(user->mis_tweets, t);
}

/**
 * Funcion que indica si un tweet pertenece al twittero 
 *
 * Entrada:
 *    El twittero y el tweet que se quiere buscar.
 *
 * Salida:
 *  TRUE si a esta en user y FALSE en caso contrario
 *    
 */
BOOL has_twittero_tweet(twittero *user, tweet *t)
{
	int i;
	int n;
	BOOL encontrado;

	assert(user != NULL && t != NULL);

	n = get_n_tweets(user->mis_tweets);
	encontrado = FALSE;

	for (i = 0; i < n && !encontrado; i++) {
		if (get_tweet_i(user->mis_tweets, i) == t) {
			encontrado = TRUE;
		}
	}

	return encontrado;
}

/**
 * Funcion para imprimir la estructura twittero por pantalla
 *
 * Entrada:
 *    Estructura twittero a imprimir
 */
void print_twittero(twittero *u)
{
	if (u == NULL) {
		printf("NULL\n");
		return;
	}

	printf("ID: %lld\n", u->id);


	printf("%s\n", u->name);
	printf("@%s\n\n", u->alias);

	print_tweets(u->mis_tweets);
}


