/**
 * Fichero : twitteros.c      Fecha:23/12/2016
 * Autores:                                   
 *                                            
 *  fichero con la implementacion del array de twitteros
 *                                            
 */

#include "twitteros.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#define TAM_BLOQUE 10

/*********************************************** Estructura del TAD twittero ***/
struct _twitteros {
	twittero **array_twitteros;
	int n_twitteros;
	int n_reserved;
	int last_id;
};

/****** Funciones internas *****/
BOOL has_id_twitteros(twitteros * users, long long int id);
BOOL has_alias(twitteros * users, const char * alias);
BOOL validate_capacity_twitteros(twitteros * users);

/**
 * Funcion para crear una nueva estructura twitteros
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura twitteros inicializado todo a vacio o NULL en caso de error
 */
twitteros * new_twitteros()
{
	twitteros *users;

	users = (twitteros *) malloc(sizeof(twitteros));

	if (users == NULL) {
		return NULL;
	}

	users-> array_twitteros = (twittero **) malloc(TAM_BLOQUE * sizeof(twittero *));
	if (users-> array_twitteros == NULL) {
		free(users);
		return NULL;
	}


	users->n_reserved = TAM_BLOQUE;
	users->last_id = 0;
	users->n_twitteros = 0;

	return users;
}

/**
 * Funcion para liberar la memoria reservada de la estructura twitteros 
 *   OJO: No se debe liberar la memoria de los twitteros del array pero si el 
 *        array
 *
 * Entrada:
 *    Estructura twitteros a liberar
 */
void destroy_twitteros(twitteros *users)
{

	assert(users);

	if (users->array_twitteros != NULL) {
		free(users->array_twitteros);
	}

	free(users);
}

/**
 * Funcion para aniadir un twittero ya creado al array. Si el alias ya existe
 *   no se puede aniadir el twittero. Tampoco si ya existe el id. En ambos casos
 *   se devuelve NULL_ID
 *
 * Entrada:
 *    Estructura del TAD twitteros sobre el que se quiere aniadir un twittero y
 *    puntero al twittero a aniadir. Si el twittero tiene el id a ID_NULO, se le
 *    asignara a last_id+1 y se incrementara last_id
 *
 * Salida:
 *    El id del twittero aniadido o NULL_ID en caso de error 
 */
long long int add_twittero(twitteros * users, twittero * user)
{
	assert(users != NULL && user != NULL);

	if (!validate_capacity_twitteros(users)) {
		return NULL_ID;
	}

	if (get_twittero_id(user) == NULL_ID) {
		set_twittero_id(user, users->last_id + 1);
		users->last_id++;
	}


	if (has_id_twitteros(users, get_twittero_id(user)) || has_alias(users, get_twittero_alias(user))) {
		return NULL_ID;
	}

	users->array_twitteros[users->n_twitteros] = user;
	users->n_twitteros++;


	return get_twittero_id(user);
}

/**
 * Funcion para recuperar el numero de twitteros del array
 *
 * Entrada:
 *    Estructura del TAD twitteros sobre el que se quiere recuperar su tamanio
 *
 * Salida:
 *    El numero de twitteros
 */
int get_n_twitteros(twitteros * users)
{
	if (users == NULL) {
		return -1;
	}

	return users->n_twitteros;
}

/**
 * Funcion para recuperar el twittero n-esimo de la lista
 *
 * Entrada:
 *    Estructura twitteros e indice del twittero a recuperar (indexado sobre 0)
 *
 * Salida:
 *    Un puntero al twittero o NULL si hay un error
 */
twittero * get_twittero_i(twitteros * users, int index)
{
	if (users == NULL || index >= users->n_twitteros || index < 0) {
		return NULL;
	}

	return users->array_twitteros[index ];

	/*Indexando sobre 1:*/
	/*
	if(users == NULL || index > users->n_twitteros || index <= 0){
	    return NULL;
	}
    
	return users->array_twitteros[index - 1];*/
}

/**
 * Funcion para recuperar un twittero dado su código.
 *
 * Entrada:
 *    Colección de twitteros y el código del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al twittero o NULL si no se encuentra.
 */
twittero * get_twittero_by_id(twitteros *users, int id)
{
	int i;
	twittero *u;

	if (users == NULL || id == NULL_ID) {
		return NULL;
	}

	u = NULL;
	for (i = 0; i < users->n_twitteros && u == NULL; i++) {
		if (get_twittero_id(users->array_twitteros[i]) == id) {
			u = users->array_twitteros[i];
		}
	}

	return u;
}

/**
 * Funcion para recuperar un twittero dado su alias.
 *
 * Entrada:
 *    Colección de twitteros y el alias del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al twittero o NULL si no se encuentra.
 */
twittero * get_twittero_by_alias(twitteros *users, const char *alias)
{
	int i;
	twittero *u;

	if (users == NULL || alias == NULL) {
		return NULL;
	}

	u = NULL;
	for (i = 0; i < users->n_twitteros && u == NULL; i++) {

		if (strcmp(get_twittero_alias(users->array_twitteros[i]), alias) == 0) {
			u = users->array_twitteros[i];
		}
	}

	return u;
}

/**
 * Funcion para imprimir la lista de twitteros por pantalla
 *
 * Entrada:
 *    Estructura TAD twittero a imprimir
 */
void print_twitteros(twitteros *users)
{
	int i;

	if (users == NULL) {
		printf("NULL\n");
		return;
	}

	printf("-----Coleccion de twitteross-----\n");
	printf("Numero de twitteros: %d\n", users->n_twitteros);
	printf("Espacio reservado para: %d twitteros\n", users->n_reserved);
	printf("Ultima ID: %d\n", users->last_id);
	printf("-----------------------------\n\n");


	for (i = 0; i < users->n_twitteros; i++) {

		print_twittero(users->array_twitteros[i]);

	}

	printf("-----------------------------\n\n");
}


/**
 * Funcion para comprobar si la id está en la colección de twitteros.
 *
 * Entrada:
 *    Colección de twitteros y la id de twitteros a comprobar
 *
 * Salida:
 *    TRUE si se encuentra la id. FALSE en caso contrario.
 */
BOOL has_id_twitteros(twitteros * users, long long int id)
{
	int i;

	assert(users != NULL && id != NULL_ID);

	for (i = 0; i < users->n_twitteros; i++) {
		if (get_twittero_id(users->array_twitteros[i]) == id) {

			return TRUE;
		}
	}

	return FALSE;


}

/**
 * Funcion para proporcionar una capacidad correcta al array antes de insertar
 * Comprueba si hay hueco para un twitterot más. Si no lo hay, redimensiona 
 * el array
 *
 * Entrada:
 *    Colección de twitteros a validar
 *
 * Salida:
 *    TRUE si la capacidad es adecuada tras su ejecución. FALSE en caso contrario
 * */

BOOL validate_capacity_twitteros(twitteros * users)
{
	twittero ** nuevo_array;

	if (users == NULL) {
		return FALSE;
	}

	if (users->n_reserved <= users->n_twitteros) {
		nuevo_array = (twittero **) realloc(users->array_twitteros, (users->n_reserved + TAM_BLOQUE) * sizeof(twittero *));

		if (nuevo_array == NULL) {

			return FALSE;
		}

		users->array_twitteros = nuevo_array;
		users->n_reserved += TAM_BLOQUE;
	}

	return TRUE;

}

/**
 * Funcion para comprobar si el alias está en la colección de twitteros.
 *
 * Entrada:
 *    Colección de twitteros y el alias de twitteros a comprobar
 *
 * Salida:
 *    TRUE si se encuentra el alias. FALSE en caso contrario.
 */
BOOL has_alias(twitteros * users, const char * alias)
{
	int i;

	assert(users != NULL && alias != NULL);


	for (i = 0; i < users->n_twitteros; i++) {
		if (strcmp(get_twittero_alias(users->array_twitteros[i]), alias) == 0) {
			return TRUE;
		}
	}

	return FALSE;
}



















